@@ -1,7 +1,7 @@
 __title__ = 'compas_fab'
 __description__ = 'Robotic fabrication package for the COMPAS Framework'
-__url__ = 'https://github.com/gramaziokohler/compas_fab'
-__version__ = '0.5.0'
+__url__ = 'https://github.com/compas-dev/compas_fab'
+__version__ = '0.6.0'
 __author__ = 'Gramazio Kohler Research'
 __author_email__ = 'gramaziokohler@arch.ethz.ch'
 __license__ = 'MIT license'
