@@ -24,6 +24,7 @@ ROS
     :nosignatures:
 
     RosClient
+    RosFileServerLoader
 
 Tasks
 -----
@@ -43,6 +44,7 @@ Exceptions
 
     BackendError
     RosError
+    RosValidationError
     VrepError
 
 """
@@ -50,11 +52,8 @@ Exceptions
 from .exceptions import *
 from .tasks import *
 from .ros.client import *
+from .ros.exceptions import *
+from .ros.fileserver_loader import *
 from .vrep.client import *
 
-from .exceptions import __all__ as a
-from .tasks import __all__ as b
-from .ros.client import __all__ as c
-from .vrep.client import __all__ as d
-
-__all__ = a + b + c + d
+__all__ = [name for name in dir() if not name.startswith('_')]
