@@ -2,7 +2,6 @@ from __future__ import absolute_import
 from __future__ import division
 from __future__ import print_function
 
-import compas
 from compas.robots import Joint
 
 __all__ = [
@@ -11,20 +10,18 @@ __all__ = [
 
 
 class Configuration(object):
-    """Represents the configuration of a robot based on the
-    state of its joints. This concept is also refered to
-    as `Joint State`.
+    """Represents the configuration of a robot based on the state of its joints.
+    This concept is also refered to as `Joint State`.
 
     Attributes
     ----------
     values : :obj:`list` of :obj:`float`
         Joint values expressed in radians or meters, depending on the respective type.
-    types : :obj:`list` of :class:`Joint.TYPE`
-        Joint types, e.g. a list of ``Joint.REVOLUTE`` for revolute joints.
+    types : list of :class:`compas.robots.Joint.TYPE`
+        Joint types, e.g. a list of `compas.robots.Joint.REVOLUTE` for revolute joints.
 
     Examples
     --------
-
     >>> from math import pi
     >>> from compas_fab.robots import Configuration
     >>> config = Configuration.from_revolute_values([pi/2, 0., 0.])
@@ -48,7 +45,8 @@ class Configuration(object):
 
     def __init__(self, values=[], types=[]):
         if len(values) != len(types):
-            raise ValueError("%d values must have %d types, but %d given." % (len(values), len(values), len(types)))
+            raise ValueError("%d values must have %d types, but %d given." % (
+                len(values), len(values), len(types)))
         self.values = list(values)
         self.types = list(types)
         self._precision = '3f'
@@ -56,7 +54,7 @@ class Configuration(object):
     def __str__(self):
         vs = '%.' + self._precision
         return "Configuration(%s, %s)" % ('(' + ", ".join([vs] * len(self.values)) % tuple(self.values) + ')', tuple(self.types))
-    
+
     def __repr__(self):
         return self.__str__()
 
@@ -97,7 +95,8 @@ class Configuration(object):
         prismatic_values = list(prismatic_values)
         revolute_values = list(revolute_values)
         values = prismatic_values + revolute_values
-        types = [Joint.PRISMATIC] * len(prismatic_values) + [Joint.REVOLUTE] * len(revolute_values)
+        types = [Joint.PRISMATIC] * \
+            len(prismatic_values) + [Joint.REVOLUTE] * len(revolute_values)
         return cls.from_data({'values': values, 'types': types})
 
     @classmethod
@@ -152,11 +151,30 @@ class Configuration(object):
     def revolute_values(self):
         """:obj:`list` of :obj:`float` : Revolute joint values in radians."""
         return [v for i, v in enumerate(self.values) if self.types[i] == Joint.REVOLUTE]
-    
+
     def copy(self):
         cls = type(self)
         return cls(self.values[:], self.types[:])
 
+    def scale(self, scale_factor):
+        """Scales the joint positions of the current configuration.
+
+        Only scalable joints are scaled, i.e. planar and prismatic joints.
+
+        Parameters
+        ----------
+        scale_factor : float
+            Scale factor
+        """
+        values_scaled = []
+
+        for value, joint_type in zip(self.values, self.types):
+            if joint_type in (Joint.PLANAR, Joint.PRISMATIC):
+                value *= scale_factor
+            values_scaled.append(value)
+
+        self.values = values_scaled
+
 
 if __name__ == "__main__":
     from math import pi
@@ -169,13 +187,16 @@ if __name__ == "__main__":
     config = Configuration.from_revolute_values([pi/2, 0., 0.])
     print(config.values)
 
-    config = Configuration.from_prismatic_and_revolute_values([8.312], [pi/2, 0., 0., 0., 2*pi, 0.8])
+    config = Configuration.from_prismatic_and_revolute_values(
+        [8.312], [pi/2, 0., 0., 0., 2*pi, 0.8])
     print(str(config))
 
-    config = Configuration([pi/2, 3., 0.1], [Joint.REVOLUTE, Joint.PRISMATIC, Joint.PLANAR])
+    config = Configuration(
+        [pi/2, 3., 0.1], [Joint.REVOLUTE, Joint.PRISMATIC, Joint.PLANAR])
     print(str(config))
 
-    config = Configuration.from_prismatic_and_revolute_values([2., 3., 1.33], map(radians, [90, 0, 0, 20]))
+    config = Configuration.from_prismatic_and_revolute_values(
+        [2., 3., 1.33], map(radians, [90, 0, 0, 20]))
     print(str(config))
     print(config.revolute_values)
     print(config.prismatic_values)
